//
//  $Id: BinaryTree.h,v 1.5 2007/04/02 04:56:05 rh Exp $
//  An Example Binary Tree Class
//
//  Created by Rene Hexel on 4/04/06.
//  Copyright 2006-2007 Rene Hexel. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SimpleTree.h"

@class BinaryTreeEnumerator;
@class BinaryTreePreorderEnumerator;
@class BinaryTreePostorderEnumerator;
@class BinaryTreeInorderEnumerator;
@class BinaryTreeBreadthFirstEnumerator;

/**
 * A simple binary tree class
 */
@interface BinaryTree: SimpleTree
{
	Class defaultEnumerator;	/** default enumerator class */
}

/**
 * Initialises a newly allocated binary tree by creating a perfectly
 * balanced tree from the objects in the argument list.
 */
- initWithObjects: (id) obj, ... ;

/**
 * Initialises a newly allocated binary tree from an array, creating a
 * perfectly balanced tree.
 */
- initWithArray: (NSArray *) array;

/**
 * Initialises a newly allocated binary tree from any collection using
 * an enumerator and a counter of the number of objects to put in the tree.
 * This creates a perfectly balanced tree.
 */
-initWithEnumerator: (NSEnumerator *) e count: (int) n;

/**
 * returns an autoreleased instance of the default binary tree
 * object enumerator
 */
- (NSEnumerator *) objectEnumerator;	// return an enumerator

/**
 * returns an autoreleased instance of a pre-order tree enumerator
 */
- (BinaryTreePreorderEnumerator *) preorderEnumerator;

/**
 * returns an autoreleased instance of a post-order tree enumerator
 */
- (BinaryTreePostorderEnumerator *) postorderEnumerator;

/**
 * returns an autoreleased instance of an in-order tree enumerator
 */
- (BinaryTreeInorderEnumerator *) inorderEnumerator;

/**
 * returns an autoreleased instance of a breadth-first tree enumerator
 */
- (BinaryTreeBreadthFirstEnumerator *) breadthFirstEnumerator;

@end

@class SimpleStack;

/**
 * An abstract object enumerator for the binary tree class
 */
@interface BinaryTreeEnumerator: TreeEnumerator
{
	id coll;			/** stack/queue for tree recursion */
}

/**
 * set the first child for the enumerator
 */
- setFirstChild: object;

/**
 * set the last child for the enumerator
 */
- setLastChild: object;

/**
 * set the first child for the enumerator
 */
- setLeftChild: object;

/**
 * set the first child for the enumerator
 */
- setRightChild: object;

/**
 * release references and deallocate memory
 */
- (void) dealloc;

@end


/**
 * A pre-order binary tree object enumerator
 */
@interface BinaryTreePreorderEnumerator: BinaryTreeEnumerator
- init;				/** default initialiser */
- advance;			/** pre-order traversal to next node */
@end

/**
 * A post-order binary tree object enumerator
 */
@interface BinaryTreePostorderEnumerator: BinaryTreeEnumerator
- init;				/** default initialiser */
- advance;			/** post-order traversal to next node */
@end

/**
 * A in-order binary tree object enumerator
 */
@interface BinaryTreeInorderEnumerator: BinaryTreeEnumerator
- init;				/** default initialiser */
- advance;			/** in-order traversal to next node */
@end

/**
 * A breadth-first binary tree object enumerator
 */
@interface BinaryTreeBreadthFirstEnumerator: BinaryTreeEnumerator
- init;				/** default initialiser */
- advance;			/** breadth-first traversal to next node */
@end
