/*
 *  $Id: BinaryTreeNode.h,v 1.3 2007/04/02 03:27:11 rh Exp $
 *  Binary Tree node for SimpleTree implementation
 *
 *  Created by René Hexel on 2/04/07.
 *  Copyright 2007 Rene Hexel. All rights reserved.
 */

#import <Foundation/Foundation.h>

/**
 * Binary Tree Node class.  This class is private to BinaryTree
 * and should only be used by that class or its subclasses!
 */
@interface _BinaryTreeNode: NSObject
{
	_BinaryTreeNode *parent;		/** parent node (nil if root) */
	_BinaryTreeNode *left;			/** left child (nil if none) */
	_BinaryTreeNode *right;			/** right child (nil if none) */
	id object;				/** contained object */
}

- parentNode;					/** return parent node */
- leftChild;					/** return left child */
- rightChild;					/** return left child */
- object;					/** return object (content) */
- setParentNode: (_BinaryTreeNode *) newParent;	/** set parent node */
- setLeftChild: (_BinaryTreeNode *) newLeft;	/** set new left child node */
- setRightChild: (_BinaryTreeNode *) newRight;	/** set new right child node */
- setObject: (id) newObj;			/** set new content */

- (void) dealloc;				/** return all memory */

/**
 * recursively count the size of the subtree starting at the current node
 */
- (unsigned) count;

/**
 * convenience method to create a perfectly balanced binary tree from
 * any collection, passing the enumerator of the collection and the
 * count of elements to be put in the tree.
 */
+ treeWithEnumerator: (NSEnumerator *) e count: (int) n;

@end
