//
//  $Id: SimpleTree.h,v 1.4 2007/04/02 03:27:11 rh Exp $
//  An Example Abstract Tree Class
//
//  Created by Rene Hexel on 4/04/06.
//  Copyright 2006-2007 Rene Hexel. All rights reserved.
//

#import <Foundation/Foundation.h>

/**
 * A simple, abstract tree class
 */
@interface SimpleTree: NSObject
{
	id root;	/** the root node of the tree */
}

/**
 * returns the default enumerator of the tree.
 */
- (NSEnumerator *) objectEnumerator;	// return an enumerator

/**
 * releases all nodes within the tree.
 */
- releaseTreeNodes;

/**
 * releases all nodes held by the tree, then deallocate tree memor
 */
- (void) dealloc;

/**
 * returns the root node of the tree.
 */
- root;

/**
 * Replace the whole tree with a new root node
 */
- setRoot: newRoot;

/**
 * count the number of nodes in the tree
 */
- (unsigned) count;

@end


/**
 * An object enumerator for the SimpleTree class
 *
 * This is an abstract class, subclasses will need to implement
 * actual enumerators.
 */
@interface TreeEnumerator: NSEnumerator
{
	SimpleTree *tree;	/** the tree to enumerate */
	id currentNode;		/** the current node */
	id nextNode;		/** the next node */
}

/**
 * returns the next, enumerated object
 */
- nextObject;

/**
 * Initialises the enumerator with a given tree.
 */
- initWithTree: (SimpleTree *) newTree;

/**
 * replaces the tree held by the enumerator with a new tree
 */
- setTree: (SimpleTree *) newTree;

/**
 * advance to the next object within the tree.
 * This method is a stub that needs to be overridden by a concrete sub class
 */
- advance;


/**
 * set the first child for the enumerator
 * This method is a stub that needs to be overridden by a concrete sub class
 */
- setFirstChild: object;

/**
 * set the last child for the enumerator
 * This method is a stub that needs to be overridden by a concrete sub class
 */
- setLastChild: object;

/**
 * release the currently held node to the Autorelease Pool
 */
- (void) autoreleaseNode;

/**
 * release the current tree reference held by the enumerator and
 * deallocate the TreeEnumerator
 */
- (void) dealloc;

@end
