//
//  $Id: SimpleTree.m,v 1.3 2007/04/02 03:14:43 rh Exp $
//  A simple abstract Tree class implementation
//
//  Copyright 2006 Rene Hexel. All rights reserved.
//

#import "SimpleTree.h"

@implementation SimpleTree

- (NSEnumerator *) objectEnumerator
{
	return [[[TreeEnumerator alloc] initWithTree: self] autorelease];
}


- releaseTreeNodes
{
	NSAutoreleasePool *pool = [NSAutoreleasePool new];
	TreeEnumerator *e = (id) [self objectEnumerator];
	while ([e nextObject]) [e autoreleaseNode];
	[pool release];
	
	return self;
}


- (void) dealloc
{
	[self releaseTreeNodes];
	[super dealloc];
}


- root
{
	return root;
}

- setRoot: newRoot
{
	[newRoot retain];
	[root release];
	root = newRoot;
	
	return self;
}


- (unsigned) count
{
	return [root count];
}

@end


/*
 * This is an abstract class, subclasses will need to implement
 * actual enumerators.  This class just provides stubs and throws
 * exceptions if they are actually used!
 */

@implementation TreeEnumerator: NSEnumerator

/*
 * This is a stub.  Subclasses need to actually implement this!
 */
- advance
{
	[NSException raise: @"AbstractClassInstantiation"
		    format: @"-advance called for %@", [self className]];

	return nil;
}


- nextObject
{
	[self advance];

	return [currentNode object];		// return contained object
}


- initWithTree: (SimpleTree *) newTree
{
	[[self init] setTree: newTree];
	nextNode = [tree root];

	return self;
}


- setTree: (SimpleTree *) newTree
{
	[tree release];
	tree = [newTree retain];

	return self;
}


- setFirstChild: object
{
	[NSException raise: @"AbstractClassInstantiation"
		    format: @"-setFirstChild: called for %@", [self className]];

	return nil;
}


- setLastChild: object
{
	[NSException raise: @"AbstractClassInstantiation"
		    format: @"-setLastChild: called for %@", [self className]];

	return nil;
}


- (void) autoreleaseNode
{
	[currentNode autorelease];
}


- (void) dealloc
{
	[tree release];
	[super dealloc];
}

@end
